#include <GL/gl.h>
#include <GL/glx.h>
#include <math.h>

#include "gla.h"
#include "o_funcs.h" 

#define FILTER_FOG	0

double ortho[6]={-1.0, 1.0, -1.0, 1.0, -1.0, 1.0};
int deform_table[3]={0, 0, 0};
double scalen[3]={1.0, 1.0, 1.0};

double defx(double x);
double defy(double y);
double defz(double z);
double defw(double w);
double effect(double f, int comp);

double sinusoide(double f, int comp);
double randomize(double f, int comp);
double tangent(double f, int comp);
double squaret(double f, int comp);
double scale(double f, int comp);

typedef struct
{
	char *name;
	double (*function)(double f, int comp);
	double *param;
} Tdef_effects;

const Tdef_effects deform_effects[]={
	{"sin",		&sinusoide,	NULL},
	{"rand",	&randomize,	NULL},
	{"tan",		&tangent,	NULL},
	{"sqrt",	&squaret,	NULL},
	{"scale",	&scale,		scalen}
};


void set_defeffect_params(const char *var)
{
		/* parse var string */
		/* format: [x=(sin|tan|rand|sqrt)*,][y=...,][z=...] */
	const char *p;
	int comp=0;
	const char comps[]={'x', 'y', 'z'};
	int s, r, i, e;
	double f;
	double sav;
	
	if(!var)
		return;
	
	for(p=var; *p!='\0'; p++) {
		switch(*p) {
		case 'x':
			comp=1;
			break;
		case 'y':
			comp=2;
			break;
		case 'z':	
			comp=3;
			break;
		case '=':
			break;
		case 's':		/* sin */
			if(comp!=0 && *(p+1)=='i' && *(p+2)=='n') {
				deform_table[comp-1]|=1;
				p+=2;
					/* sqrt */
			} else if(comp!=0 && *(p+1)=='q' && *(p+2)=='r' && *(p+3)=='t') {
				deform_table[comp-1]|=8;
				p+=3;
			} else if(comp!=0 && *(p+1)=='c' && *(p+2)=='a' && *(p+3)=='l' && *(p+4)=='e' && *(p+5)=='=') {
				s=1;
				sav=deform_effects[4].param[comp-1];
				deform_effects[4].param[comp-1]=0;
				f=0.1;
				p+=6;
				r=0;
				if((*p<' ' || *p>'9') && *p!='.')
					perror("invalid syntax in variable GLA_DEFORM_EFFECTS");
					/* read float number */
				for(; (*p>=' ' && *p<='9') || *p=='.'; p++, r++) {
					if(*p=='.') {
						if(s==-1)
							perror("invalid syntax in variable GLA_DEFORM_EFFECTS");
						s=-1;
					} else {
						if(s>0) {
							deform_effects[4].param[comp-1]*=10;
							deform_effects[4].param[comp-1]+=*p-'0';
						} else {
							deform_effects[4].param[comp-1]+=f*(*p-'0');
							f*=0.1;
						}
					}
				}
				if(!r)
					deform_effects[4].param[comp-1]=sav;
				else
					printf("param read: %f\n", deform_effects[4].param[comp-1]);
				deform_table[comp-1]|=16;
				p--;
			} else
				perror("invalid syntax in variable GLA_DEFORM_EFFECTS");
			break;
		case 't':		/* tan */
			if(comp!=0 && *(p+1)=='a' && *(p+2)=='n') {
				deform_table[comp-1]|=4;
				p+=2;
			} else
				perror("invalid syntax in variable GLA_DEFORM_EFFECTS");
			break;
		case 'r':		/* rand */
			if(comp!=0 && *(p+1)=='a' && *(p+2)=='n' && *(p+3)=='d') {
				deform_table[comp-1]|=2;
				p+=2;
			} else
				perror("invalid syntax in variable GLA_DEFORM_EFFECTS");
			break;
		case ',':
			comp=0;
			break;
		case '|':
			break;
		default:
			perror("invalid syntax in variable GLA_DEFORM_EFFECTS");
		}
	}
	printf("gla deformations: \n");
	for(comp=0; comp<3; comp++) {
		printf("%c: ", comps[comp]);
		for(i=1, e=0; i<=16; i*=2, e++)
			if(deform_table[comp] & i)
				printf("%s", deform_effects[e].name);
		printf("\n");
	}
}

void glOrtho(GLdouble left, GLdouble right, GLdouble bottom, GLdouble top, GLdouble near_val, GLdouble far_val)
{
	ortho[0]=left;
	ortho[1]=right;
	ortho[2]=bottom;
	ortho[3]=top;
	ortho[4]=near_val;
	ortho[5]=far_val;
	o_glOrtho(left, right, bottom, top, near_val, far_val);
}

	/* aliases functions */

void glVertex2f( GLfloat x, GLfloat y )
{
	glVertex2d(x, y);
}

void glVertex2i( GLint x, GLint y )
{
	glVertex2d(x, y);
}

void glVertex2s( GLshort x, GLshort y )
{
	glVertex2d(x, y);
}

void glVertex3f( GLfloat x, GLfloat y, GLfloat z )
{
	glVertex3d(x, y, z);
}

void glVertex3i( GLint x, GLint y, GLint z )
{
	glVertex3d(x, y, z);
}

void glVertex3s( GLshort x, GLshort y, GLshort z )
{
	glVertex3d(x, y, z);
}

void glVertex4f( GLfloat x, GLfloat y, GLfloat z, GLfloat w )
{
	glVertex4d(x, y, z, w);
}

void glVertex4i( GLint x, GLint y, GLint z, GLint w )
{
	glVertex4d(x, y, z, w);
}

void glVertex4s( GLshort x, GLshort y, GLshort z, GLshort w )
{
	glVertex4d(x, y, z, w);
}

void glVertex2dv( const GLdouble *v )
{
	glVertex2d(v[0], v[1]);
}

void glVertex2fv( const GLfloat *v )
{
	glVertex2dv(v);
}

void glVertex2iv( const GLint *v )
{
	glVertex2dv(v);
}

void glVertex2sv( const GLshort *v )
{
	glVertex2dv(v);
}

void glVertex3dv( const GLdouble *v )
{
	glVertex3d(v[0], v[1], v[2]);
}

void glVertex3fv( const GLfloat *v )
{
	glVertex3dv(v);
}

void glVertex3iv( const GLint *v )
{
	glVertex3dv(v);
}

void glVertex3sv( const GLshort *v )
{
	glVertex3dv(v);
}

void glVertex4dv( const GLdouble *v )
{
	glVertex4d(v[0], v[1], v[2], v[3]);
}

void glVertex4fv( const GLfloat *v )
{
	glVertex4dv(v);
}

void glVertex4iv( const GLint *v )
{
	glVertex4dv(v);
}

void glVertex4sv( const GLshort *v )
{
	glVertex4dv(v);
}


	/* main function to be modified, called by all the others */
void glVertex4d( GLdouble x, GLdouble y, GLdouble z, GLdouble w )
{
	o_glVertex4d(defx(x), defy(y), defz(z), defw(w));
}


void glVertex3d( GLdouble x, GLdouble y, GLdouble z )
{
	o_glVertex3d(defx(x), defy(y), defz(z));
}

void glVertex2d( GLdouble x, GLdouble y )
{
	o_glVertex2d(defx(x), defy(y));
}

	/* deformation functions */
double defx(double x)
{
	if(!deform_table[0])
		return x;
	return effect(x, 0);
}

double defy(double y)
{
	if(!deform_table[1])
		return y;
	return effect(y, 1);
}

double defz(double z)
{
	if(!deform_table[2])
		return z;
	return effect(z, 2);
}

double defw(double w)
{
		/* no deformation on w */
	return w;
}

double effect(double f, int comp)
{
	int i, e;
	double r=f;
	
	for(i=1, e=0; i<=16; i*=2, e++)
		if(deform_table[comp] & i) {
/*printf("comp: %d: %s\n", comp, deform_effects[e].name);*/
			r=deform_effects[e].function(r, comp);
		}
	return r;
}

	/* these effects are done using a scale varying from 0.75 to 1.25 */

double sinusoide(double f, int comp)
{
	return ((sin(f)+1.0)/4+0.75)*f;
/*	return (sin(f*2.0)/2.0)+f;*/
}

double cosine(double f, int comp)
{
	return ((cos(f)+1.0)/4+0.75)*f;
/*	return (cos(f*2.0)/2.0)+f;*/
}

double randomize(double f, int comp)
{
	return ((random()/(double)(RAND_MAX))/2+0.75)*f;
/*	return ((random()/(double)RAND_MAX) * 0.4)+f;*/
}

double tangent(double f, int comp)
{
	return tanf(f);
}

double squaret(double f, int comp)
{
	return f<0.0?(-sqrt(-f)):(sqrt(f));
}

double scale(double f, int comp)
{
	return f*scalen[comp];
}
