#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/keysym.h>
#include <assert.h>
#include <string.h>
#include <ctype.h>

#include "o_funcs.h"
#include "gla.h"

#define MAX_KEYS	64


int grab=0;	// discard all input for the running program
extern int currmenu;
extern int active_color;
extern char float_str[];


static int key_result(unsigned int key) {
	char *str=XKeysymToString(key);
	// key must be alpha or num or . or backspace or enter
	if (key==XK_Return)
		run_menu('\n');
	else if (key==XK_period)
		run_menu('.');
	else if ( (strlen(str)==1 && (isalnum(str[0]))) || key==XK_period || key==XK_BackSpace || key==XK_Return )
		run_menu(toupper(str[0]));
}

static Bool key_pressed_check(Display * display, XEvent *ev, char *keycode_val) {
	XKeyPressedEvent *keyev;
	char buf[64];
	KeySym keysym;
	XComposeStatus *status=NULL;

	if (ev->type != KeyPress)
		return 0;
	keyev = (XKeyEvent *) ev;

	/* take only menu activator */
	if (keyev->keycode==76) /* =F10 */ {
		printf("menu activation/desactivation\n");
		grab=!grab;
		active_color=-1;
		currmenu=-1;
		float_str[0]='\0';
		return 1;
	} 

	/* if in menu mode, attempt to stack the key */
	if (grab) {
		XLookupString(keyev, buf, 64, &keysym, status);
		printf("reading key :\"%s\"\n", XKeysymToString(keysym));
		key_result(keysym);
		return 1;
	}

	/* if not returned before, this key is not removed from X buffer */
	return 0;
}

void cleanQueue(Display *display) {
	static XEvent ev;
	assert(o_XCheckIfEvent);

	while (o_XCheckIfEvent(display, &ev, key_pressed_check, NULL));
}

int XNextEvent(Display *display, XEvent *event_return) {
	cleanQueue(display);
	o_XNextEvent(display, event_return);
	return 0;
}

int XPeekEvent(Display *display, XEvent *event_return) {
	cleanQueue(display);
	o_XPeekEvent(display, event_return);
	return 0;
}
int XWindowEvent(Display *display, Window w, long event_mask, XEvent *event_return) {
	cleanQueue(display);
	o_XWindowEvent(display, w, event_mask, event_return);
	return 0;
}
Bool XCheckWindowEvent(Display *display, Window w, long event_mask, XEvent *event_return) {
	cleanQueue(display);
	return o_XCheckWindowEvent(display, w, event_mask, event_return);
}
int XMaskEvent(Display *display, long event_mask, XEvent *event_return) {
	cleanQueue(display);
	o_XMaskEvent(display, event_mask, event_return);
	return 0;
}
Bool XCheckMaskEvent(Display *display, long event_mask, XEvent *event_return) {
	cleanQueue(display);
	return o_XCheckMaskEvent(display, event_mask, event_return);
}
Bool XCheckTypedEvent(Display *display, int event_type, XEvent *event_return) {
	cleanQueue(display);
	return o_XCheckTypedEvent(display, event_type, event_return);
}
Bool XCheckTypedWindowEvent(Display *display, Window w, int event_type, XEvent *event_return) {
	cleanQueue(display);
	return o_XCheckTypedWindowEvent(display, w, event_type, event_return);
}
int XIfEvent(Display *display, XEvent *event_return, Bool (*predicate)(), XPointer arg) {
	cleanQueue(display);
	o_XIfEvent(display, event_return, predicate, arg);
	return 0;
}

Bool XCheckIfEvent(Display *display, XEvent *event_return,Bool (*predicate)() , XPointer arg) {
	cleanQueue(display);
	return o_XCheckIfEvent(display, event_return, predicate , arg);
}

int XPeekIfEvent(Display *display, XEvent *event_return, Bool (*predicate)(), XPointer arg) {
	cleanQueue(display);
	o_XPeekIfEvent(display, event_return, predicate, arg);
	return 0;
}


int XPending(Display *display){
	cleanQueue(display);
	return o_XPending(display);
}

int XEventsQueued(Display *display, int mode) {
	cleanQueue(display);
	return o_XEventsQueued(display, mode);
}



